#include "plumpconfig.h"

int plump_open(int pid, unsigned long access, plumpmedium_t medium, plump_t **ret)
{
	int retval = PLUMP_SUCCESS;

	if (!ret)
		return PLUMP_ERROR_BADPTR;

	*ret = NULL;

	// Is the medium even supported?
	if (plump_mediumIsSupported(medium) == PLUMP_ERROR_NOTSUPPORTEDMEDIUM)
		return PLUMP_ERROR_NOTSUPPORTEDMEDIUM;

	do
	{
		// Allocate the context
		if (!(*ret = (plump_t *)malloc(sizeof(plump_t))))
		{
			retval = PLUMP_ERROR_NOMEMORY;
			break;
		}

		// Zero out the plump_t
		memset(*ret, 0, sizeof(plump_t));

		// Get the vftable for the given medium
		if ((retval = plump_mediumGetFunctionTable(medium, &(*ret)->vftable)) != PLUMP_SUCCESS)
			break;

		// Do the underlying medium open
		if ((retval = (*ret)->vftable.open(*ret, pid, access)))
			break;

		(*ret)->pid    = pid;
		(*ret)->access = access;
		(*ret)->medium = ((*ret)->vftable.getMedium) ? (*ret)->vftable.getMedium() : PLUMP_MEDIUM_ANY;

		retval = PLUMP_SUCCESS;

	} while (0);

	if (retval != PLUMP_SUCCESS)
	{
		plump_close(*ret);

		*ret = NULL;
	}

	return retval;
}

int plump_getMedium(plump_t *p, plumpmedium_t *medium)
{
	if (medium)
		*medium = p->medium;

	return PLUMP_SUCCESS;
}

int plump_close(plump_t *p)
{
	if (!p)
		return PLUMP_ERROR_BADPTR;

	if (p->vftable.close)
		p->vftable.close(p);

	free(p);

	return PLUMP_SUCCESS;
}

